SystemOrganization addCategory: #'Math-Random'!

Object subclass: #MarsagliaKissRandomKernel
	instanceVariableNames: 'jcong jsr w z'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random'!

!MarsagliaKissRandomKernel commentStamp: 'nice 4/21/2014 16:29' prior: 0!
A MarsagliaKissRandomKernel is a private class for generating pseudo-Random numbers.

It generates 32-bits Integer with uniform distribution in Interval [ 0,16rFFFFFFFF].

It holds the states of the pseudo-random generator, and the kernel generation algorithm.
The algorithm is the one used in libgfortran library.
It is based on Marsaglia's Keep It Simple Stupid 2005 version as in "double precision RNGs" in  sci.math.num-analysis  
http://sci.tech-archive.net/Archive/sci.math.num-analysis/2005-11/msg00352.html

It is a combination of:
1) a linear congruential generator with period 2^32
2) a 3-shift shift-register generator of period 2^32-1
3) 2 16-bit multiply with carry generators with a period 597273182964842497 > 2^59

Period of this generator is about 2^123

Previous 1999 version can be found along with discussions in this sci.stat.math newsgroup archive:
http://www.ciphersbyritter.com/NEWS4/RANDC.HTM#369F6FCA.74C7C041@stat.fsu.edu
The shift-register had a permutation of first two shifts (13 and 17) leading to a reduced period.

Warning: this pseudo-random generator is not suitable for cryptography as it could be too easily broken (see http://eprint.iacr.org/2011/007.pdf)

Instance Variables
	jcong:	<Integer> state of the linear congruencial generator
	jsr:		<Integer> state of the 3-shift register generator
	w:		<Integer> state of the first multiply with carry generator
	z:		<Integer> state of the second multiply with carry generator
!

!MarsagliaKissRandomKernel class methodsFor: 'instance creation' stamp: 'nice 4/25/2014 01:31'!
new
	| seedGenerator |
	seedGenerator := self seed: ((1 to: 4) collect: [:i | Time millisecondClockValue + i]).
	^self seed: (seedGenerator next: 4)! !

!MarsagliaKissRandomKernel class methodsFor: 'instance creation' stamp: 'nice 4/21/2014 17:11'!
seed: anArrayWithFourIntegers
	"Anwer a new Random stream with the initial seed anArrayWithFourIntegers."

	^ self basicNew seed: anArrayWithFourIntegers; yourself! !

!MarsagliaKissRandomKernel methodsFor: 'initialize-release' stamp: 'nice 4/21/2014 16:14'!
initialize
	"Do nothing - instances are initialized thru seed:"
	^self! !

!MarsagliaKissRandomKernel methodsFor: 'accessing' stamp: 'nice 4/21/2014 17:04'!
next
	"Anwser a 32 bits positive integer in [0,16rFFFFFFFF]."
	jcong := jcong * 69069 + 1327217885 bitAnd: 16rFFFFFFFF.
	jsr := (jsr bitAnd: 16r0007FFFF) << 13 bitXor: jsr.
	jsr := jsr >> 17 bitXor: jsr.
	jsr := (jsr bitAnd: 16r07FFFFFF) << 5 bitXor: jsr.
	w := (w bitAnd: 16rFFFF) * 18000 + (w >> 16).
	z := (z bitAnd: 16rFFFF) * 30903 + (z >> 16).
	^jcong + jsr + ((w bitAnd: 16rFFFF) << 16) + z bitAnd: 16rFFFFFFFF! !

!MarsagliaKissRandomKernel methodsFor: 'accessing' stamp: 'nice 4/21/2014 17:09'!
next: anInteger
	" Answer an Array of anInteger pseudo-random 
	32 bits integers between 0 and 16rFFFFFFFF. "

	^ ( 1 to: anInteger ) 
			collect: [ :i | self next ]! !

!MarsagliaKissRandomKernel methodsFor: 'accessing' stamp: 'nice 4/21/2014 16:07'!
peek
	^self copy next! !

!MarsagliaKissRandomKernel methodsFor: 'accessing' stamp: 'nice 4/21/2014 16:05'!
seed
	^(WordArray new: 4)
		with: jcong
		with: jsr
		with: w
		with: z! !

!MarsagliaKissRandomKernel methodsFor: 'accessing' stamp: 'nice 4/21/2014 18:36'!
seed: seedArray
	jcong := (seedArray at: 1) bitAnd: 16rFFFFFFFF.
	jsr := (seedArray at: 2) bitAnd: 16rFFFFFFFF.
	w := (seedArray at: 3) bitAnd: 16rFFFFFFFF.
	z := (seedArray at: 4) bitAnd: 16rFFFFFFFF.! !

Object subclass: #NumberGenerator
	instanceVariableNames: 'rng'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random'!

!NumberGenerator commentStamp: '<historical>' prior: 0!
A NumberGenerator is a stream of numbers. All NumberGenerators respond to #next.!

NumberGenerator subclass: #BernoulliGenerator
	instanceVariableNames: 'p'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random'!

!BernoulliGenerator commentStamp: '<historical>' prior: 0!
A BernoulliGenerator is simulates a Bernoulli Process. This is a discrete process, with probability of p for success, and 1-p for failure. 

next
	answer 1 if success event, 0 otherwise
	
generator:
	provide a uniform [0,1] random number generator
	
p: 
	set the probability of success events
	
class methods

withProbability:
	create a generator with probability set to p
	
defaultGenerator
	class used for generator in new instances!

!BernoulliGenerator class methodsFor: 'instance creation' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
defaultGenerator
	^ PMRandom! !

!BernoulliGenerator class methodsFor: 'instance creation' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
fair
	^ self withProbability: 0.5! !

!BernoulliGenerator class methodsFor: 'private' stamp: 'SergeStinckwich 3/10/2015 13:20'!
validProbability: aProbability
	"reject probabilities outside of [0,1]"

	self assert: [ (aProbability < 0) not and: (aProbability > 1) not ]! !

!BernoulliGenerator class methodsFor: 'instance creation' stamp: 'SergeStinckwich 3/10/2015 13:20'!
withProbability: p
	self validProbability: p.
	^ self new p: p! !

!BernoulliGenerator methodsFor: 'initialize-release' stamp: 'SergeStinckwich 3/10/2015 13:20'!
initialize
	super initialize.
	self generator: self class defaultGenerator new.
	self p: 0.5! !

!BernoulliGenerator methodsFor: 'accessing' stamp: 'SergeStinckwich 3/10/2015 13:19'!
next
	rng next < p
		ifTrue: [ ^ 1 ]
		ifFalse: [ ^ 0 ]! !

!BernoulliGenerator methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
p
	^ p! !

!BernoulliGenerator methodsFor: 'accessing' stamp: 'SergeStinckwich 3/10/2015 13:19'!
p: aProbability
	BernoulliGenerator validProbability: aProbability.
	p := aProbability! !

NumberGenerator subclass: #BinomialGenerator
	instanceVariableNames: 'n p'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random'!

!BinomialGenerator commentStamp: '<historical>' prior: 0!
A BinomialGenerator yields results from a binomial distribution with probability p and n trials. The generator is the underlying random source.

If each independent event has probability 0<p<1, and n trials are performed, next returns number of successes.!

!BinomialGenerator class methodsFor: 'instance-creation' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
n: numberOfTrials p: probabilityOfSuccess
	^ (self new ) n: numberOfTrials; p: probabilityOfSuccess.! !

!BinomialGenerator methodsFor: 'accessing' stamp: 'SergeStinckwich 3/10/2015 13:20'!
initialize
	self generator: PMRandom new.
	self n: 10; p: 0.5! !

!BinomialGenerator methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
n: numberOfTrials 
	n := numberOfTrials! !

!BinomialGenerator methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
next
	| x |
	x := 0.
	n timesRepeat: [ rng next <= p ifTrue: [ x := x + 1 ] ].
	^ x! !

!BinomialGenerator methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
p: aProbability 
	p := aProbability! !

NumberGenerator subclass: #ConstantGenerator
	instanceVariableNames: 'constant'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random'!

!ConstantGenerator commentStamp: 'SergeStinckwich 3/30/2010 15:31' prior: 0!
A ConstantGenerator is not really a random number generator.!

!ConstantGenerator class methodsFor: 'instance-creation' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
constant: aNumber
	^ self new constant: aNumber ! !

!ConstantGenerator methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
constant: aConstant
	constant := aConstant! !

!ConstantGenerator methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
generator: aRandomGenerator
	"Do nothing."
	^ self! !

!ConstantGenerator methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
next
	^ constant! !

NumberGenerator subclass: #ExponentialGenerator
	instanceVariableNames: 'mean'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random'!

!ExponentialGenerator commentStamp: '<historical>' prior: 0!
An ExponentialGenerator uses a uniform random variable in [0,1] to sample from an exponential distribution.

The exponential distribution has a single parameter beta, here denoted as mean. 

The default RandomGenerator is PMRandom, but can be modified.

The next method uses the formula:

x=  - \beta * ln (1 - u)

to generate an exponential sample x from a uniform [0,1] sample u.

(Applied Statistics 3rd ed., Ledolter and Hogg, p. 185)!

!ExponentialGenerator class methodsFor: 'instance creation' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
defaultGenerator
	^ PMRandom! !

!ExponentialGenerator class methodsFor: 'instance creation' stamp: 'SergeStinckwich 3/10/2015 13:21'!
mean: aNumber
	^ (self basicNew) 
		generator: (self defaultGenerator new);
		mean: aNumber! !

!ExponentialGenerator class methodsFor: 'instance creation' stamp: 'SergeStinckwich 3/10/2015 13:21'!
new
	^ (self basicNew) 
		generator: (self defaultGenerator new);
		mean: 1! !

!ExponentialGenerator methodsFor: 'as yet unclassified' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
mean
	^ mean! !

!ExponentialGenerator methodsFor: 'as yet unclassified' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
mean: aNumber
	mean := aNumber! !

!ExponentialGenerator methodsFor: 'as yet unclassified' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
next
	^ mean negated * ((1 - rng next) ln)! !

!ExponentialGenerator methodsFor: 'as yet unclassified' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
peek
	^ mean negated * ((1 - rng peek) ln)! !

NumberGenerator subclass: #GaussianGenerator
	instanceVariableNames: 'mean standardDeviation nextGaussian'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random'!

!GaussianGenerator commentStamp: '<historical>' prior: 0!
A GaussianGenerator is the Normal Distribution.

Instance Variables
	hasNextGaussian:		<Object>
	mean:		<Object>
	nextGaussian:		<Object>
	rng:		<Object>
	standardDeviation:		<Object>

hasNextGaussian
	- xxxxx

mean
	- xxxxx

nextGaussian
	- xxxxx

rng
	- xxxxx

standardDeviation
	- xxxxx
!

!GaussianGenerator methodsFor: 'as yet unclassified' stamp: 'HwaJong 1/7/2015 16:32'!
hasNextGaussian
	^nextGaussian notNil! !

!GaussianGenerator methodsFor: 'initialize-release' stamp: 'SergeStinckwich 3/10/2015 13:21'!
initialize
	super initialize.
	self mean: 0.
	self standardDeviation: 1! !

!GaussianGenerator methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
mean: aMeanValue
	mean := aMeanValue! !

!GaussianGenerator methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
next
	^ self nextGaussianNormalized * standardDeviation + mean! !

!GaussianGenerator methodsFor: 'accessing' stamp: 'HwaJong 1/7/2015 16:31'!
nextGaussianNormalized
	| x1 x2 w y1 y2 |
	self hasNextGaussian
		ifTrue: [ 
			[^ nextGaussian] ensure: [ nextGaussian := nil ] ].
	[ 
	x1 := 2.0 * rng nextFloat - 1.0.
	x2 := 2.0 * rng nextFloat - 1.0.
	w := x1 * x1 + (x2 * x2).
	w >= 1.0 ] whileTrue.
	w := (-2.0 * w ln / w) sqrt.
	y1 := x1 * w.
	y2 := x2 * w.
	nextGaussian := y2.
	^ y1! !

!GaussianGenerator methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
standardDeviation: aStandardDeviation
	standardDeviation := aStandardDeviation! !

!NumberGenerator class methodsFor: 'factory' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
defaultGenerator
	^ PMRandom! !

!NumberGenerator class methodsFor: 'instance-creation' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
new
	^ ((self  basicNew) generator: self defaultGenerator new) initialize; yourself! !

!NumberGenerator methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
generator
	^ rng! !

!NumberGenerator methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
generator: aRandomGenerator
	rng := aRandomGenerator ! !

!NumberGenerator methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
next
	^ self subclassResponsibility! !

!NumberGenerator methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
peek
	"really a subclass responsibility, this may be overriden whenever its more efficient to do so"
	^ (self copy generator: (self generator copy)) next! !

NumberGenerator subclass: #PoissonGenerator
	instanceVariableNames: 'lambda'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random'!

!PoissonGenerator commentStamp: '<historical>' prior: 0!
A PoissonGeneratorsimulates a Poisson Process with parameter lambda.

This class is implemented to yield an integer result, corresponding to the number of events in n intervals, with probabilility of an event in each individual interval of p. The parameter lambda is the product of n and p.!

!PoissonGenerator class methodsFor: 'instance creation' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
defaultGenerator
	^ PMRandom! !

!PoissonGenerator class methodsFor: 'instance creation' stamp: 'SergeStinckwich 3/10/2015 13:21'!
lambda: aNumber
	^ self basicNew
		generator: self defaultGenerator new;
		lambda: aNumber! !

!PoissonGenerator class methodsFor: 'instance creation' stamp: 'SergeStinckwich 3/10/2015 13:21'!
new
	^ self basicNew
		generator: self defaultGenerator new;
		lambda: 1! !

!PoissonGenerator methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
lambda
	^ lambda! !

!PoissonGenerator methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
lambda: anObject
	lambda := anObject.
	! !

!PoissonGenerator methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
next
	"generator based on multiplication of uniform random variable.
	Luc DeVroye, Non-Uniform Random Variate Generation, Section X.3
	"
	| x prod limit |
	limit := self lambda negated exp.
	x := 0.
	prod := 1.
	[true] whileTrue:
		[ | u |
		u := self generator next.
		prod := u * prod.
		(prod > limit) 
			ifTrue: [x := x + 1] 
			ifFalse: [^ x]].! !

Object subclass: #RandomGenerator
	instanceVariableNames: 'seed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random'!

!RandomGenerator commentStamp: 'hfm 3/23/2009 21:02' prior: 0!
This is the top abstract class of the pseudo-random numbers generator.

The applications of PRNG's are many ones:
-Simulations
-Sampling
-Cryptography
-Numeric Analysis
-Games
-Aesthetics

Internally, one can classify RNG's according to several views, so choosing one or another is fully dependent of your requirements:

-From the production view, there is always a tradeoff between:
--Fast or
--Secure

Another view, based on the seed they use, is:
-Deterministic: They take the seed value from a specific number. These ones are called "Pseudo random number generators"
-Non-deterministic: They take the seed value from a physical source non-predictable and outside the human control. These are called "true Random number generators"

Along these terms, deterministic RNG's are divided between:
-Normal PRNG's
--Linear Congruential
--Non-Linear Congruential
-Cryptography safe PRNG's (CSPRNG's)
--DSA
--ECDSA

And finally, there's a taxonomy more related with the internal implementation, based on how the pseudo-generator make its random variables following a distribution function.


!

RandomGenerator subclass: #LCRandom
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random'!

!LCRandom commentStamp: 'hfm 3/23/2009 21:22' prior: 0!
This uses a quick and dirty Linear Congruential generator.

LCRandom new next.!

!LCRandom methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
next
	"Private - Answer a pseudo-Random floating point number
	between 0 and 1.
	Uses a quick and dirty Linear congruential generator,"
	^ self nextFloat! !

!LCRandom methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
nextFloat
    "Private - Answer a pseudo-Random floating point number between 0 and 1.
    Uses a quick and dirty Linear congruential generator,"

    seed := self peekSeed.
    ^ seed / 120050.0! !

!LCRandom methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
peek
    "Private - Answer a pseudo-Random floating point number between 0 and 1.
    Uses a simple, but fast, Linear Congruential generator."

    ^ self peekSeed / 120050.0! !

!LCRandom methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
peekSeed
    "Private - Answer the next Integer seed calculated 
	using a quick and dirty Linear congruential generator."

    ^ seed * 2311 + 25367 \\ 120050! !

RandomGenerator subclass: #LehmerRandom
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random'!

!LehmerRandom commentStamp: 'hfm 3/23/2009 21:22' prior: 0!
This generator uses the Lehmer's Linear Congruential method.

LehmerRandom new next!

!LehmerRandom methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
next
    "Private - Answer a Random Float between 0 and 1. 
	Uses Lehmer's linear congruential method."

    ^ self nextFloat! !

!LehmerRandom methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
nextFloat
    "Private - Answer a Random Float between 0 and 1. Uses Lehmer's linear congruential method."

    seed := self peekSeed.
    ^ seed / SmallInteger maxVal asFloat! !

!LehmerRandom methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
peek
    "Private - Answer a pseudo-Random floating point number between 0 and 1."

    ^ self peekSeed / SmallInteger maxVal asFloat! !

!LehmerRandom methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
peekSeed
    "Private - Answer the next Integer seed calculated using Lehmer's linear congruential
    method. The seed is in the range 0..16r3FFFFFFF."

    ^ ( seed * 31415821 ) + 1 bitAnd: SmallInteger maxVal! !

RandomGenerator subclass: #MarsagliaKissRandom
	instanceVariableNames: 'kernelRand1 kernelRand2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random'!

!MarsagliaKissRandom commentStamp: 'nice 4/21/2014 21:44' prior: 0!
A MarsagliaKissRandom is a pseudo-random number generator (PRNG) using Marsaglia's Keep it Simple Stupid algorithm.

It generates Float with uniform distribution in Interval [0,1).
The result divided by 1.0 predecessor will be in [0,1].

Instance Variables
	kernelRand1:		<MarsagliaKissRandomKernel> a first generator
	kernelRand2:		<MarsagliaKissRandomKernel> a second generator
!

!MarsagliaKissRandom class methodsFor: 'instance creation' stamp: 'nice 4/21/2014 16:36'!
default
	^self seed: #(
		123456789 362436069 521288629 316191069
		987654321 458629013 582859209 438195021)! !

!MarsagliaKissRandom class methodsFor: 'instance creation' stamp: 'nice 4/21/2014 16:41'!
new
	| seedGenerator |
	seedGenerator := MarsagliaKissRandomKernel new.
	^self seed: (seedGenerator next: 8)! !

!MarsagliaKissRandom methodsFor: 'accessing' stamp: 'nice 4/21/2014 16:34'!
next
	"Answer a Float in interval [0.0,1.0) with uniform distribution.
	Note that constant 16rFFFFF800 is computed so as to truncate the 64 bits to Float precision.
	It is thus ((1 << 32 - 1 << (64 - Float precision) bitAnd: 1 << 32 - 1)) hex"
	^(kernelRand1 next asFloat timesTwoPower: -32)
		+ ((kernelRand2 next bitAnd: 16rFFFFF800) asFloat timesTwoPower: -64)! !

!MarsagliaKissRandom methodsFor: 'accessing' stamp: 'nice 4/21/2014 16:31'!
peek
	^self copy next! !

!MarsagliaKissRandom methodsFor: 'copying' stamp: 'SergeStinckwich 3/10/2015 13:22'!
postCopy
	kernelRand1 := kernelRand1 copy.
	kernelRand2 := kernelRand2 copy! !

!MarsagliaKissRandom methodsFor: 'accessing' stamp: 'nice 4/21/2014 16:23'!
seed
	^ kernelRand1 seed , kernelRand2 seed! !

!MarsagliaKissRandom methodsFor: 'accessing' stamp: 'SergeStinckwich 3/10/2015 13:22'!
seed: aWordArray
	"Initialize with an Array of eight 32-bits Integer"
	kernelRand1 := MarsagliaKissRandomKernel seed: (aWordArray first: 4).
	kernelRand2 := MarsagliaKissRandomKernel seed: (aWordArray last: 4)! !

RandomGenerator subclass: #MersenneTwisterRandom
	instanceVariableNames: 'states mti'
	classVariableNames: 'DefaultLengthVector Mt19937LowerMask Mt19937Mask Mt19937MatrixA Mt19937UpperMask PeriodParameter TemperingMaskB TemperingMaskC'
	poolDictionaries: ''
	category: 'Math-Random'!

!MersenneTwisterRandom commentStamp: 'hfm 3/24/2009 08:27' prior: 0!
See http://www.math.sci.hiroshima-u.ac.jp/~m-mat/MT/emt.html

" To get a pseudo-random float "
MersenneTwisterRandom new next. 

" To get a pseudo-random integer "
MersenneTwisterRandom new nextInteger.

!

!MersenneTwisterRandom class methodsFor: 'class initialization' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
initialize
	" self initialize "

	DefaultLengthVector := 624.
	Mt19937LowerMask := 2147483647.
	Mt19937Mask := 4294967295.
	Mt19937MatrixA := 16r9908B0DF.	" 2567483615 "
	Mt19937UpperMask := 16r80000000. " 2147483648 "
	PeriodParameter := 397.
	TemperingMaskB := 16r9D2C5680.	" 2636928640 "
	TemperingMaskC := 16rEFC60000.  " 4022730752 "

! !

!MersenneTwisterRandom methodsFor: 'initialize-release' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
initialize
	" Private - Initialize the receiver state vector and multiplier "

	states := Array new: DefaultLengthVector.
	mti := 1.

	states at: 1 put: ( self seed bitAnd: Mt19937Mask ).
		
	2 to: DefaultLengthVector
		do: [: multIndex | 
			states at: multIndex 
				put: ( 16r6C078965 * 
						( ( states at: multIndex - 1 ) 
							bitXor: ( ( states at: multIndex - 1 ) bitShift: 30 negated ) ) + multIndex ).
								
			states at: multIndex 
				put: ( ( states at: multIndex ) bitAnd: Mt19937Mask )
      ]
! !

!MersenneTwisterRandom methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
next
    "Private - Answer a pseudo-Random float number
	four times faster than rand() ff period 2**19937-1.  "

   ^ self nextFloat
! !

!MersenneTwisterRandom methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
nextFloat
  " Answer a random float number in the interval [ 0 , 1 ] "

  ^ self nextInteger * ( 1.0 / (4294967295.0 * 2 ))! !

!MersenneTwisterRandom methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
nextFloat: limit
  " Answer a random float number in the interval [ 0 , limit ] "

  ^ self nextFloat * limit! !

!MersenneTwisterRandom methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
nextFloatExcludeUpper
  " Answer a random float number in the interval [ 0 , 1 ) "

  ^ self nextInteger * ( 1.0 / (4294967296.0 * 2) )! !

!MersenneTwisterRandom methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
nextFloatExcludeUpper: limit
  " Answer a random float number in the interval [ 0 , limit ) "

  ^ self nextFloatExcludeUpper * limit! !

!MersenneTwisterRandom methodsFor: 'private' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
nextInteger
	" Answer a random integer on [0,0xffffffff] interval "

	| tempArray y |
	tempArray := Array withAll: #(0 16r9908B0DF).
	mti >= DefaultLengthVector
		ifTrue: [ 
			mti = (DefaultLengthVector + 1)
				ifTrue: [ 
					self seed: 5489.
					self initialize ].
			1 to: DefaultLengthVector - PeriodParameter do: [ :kk | 
				y := ((states at: kk) bitAnd: Mt19937UpperMask) bitOr: ((states at: kk + 1) bitAnd: Mt19937LowerMask).
				states
					at: kk
					put:
						(((states at: kk + PeriodParameter) bitXor: (y bitShift: 1 negated)) bitXor: (tempArray at: (y bitAnd: 16r1) + 1)) ].
			DefaultLengthVector - PeriodParameter + 1 to: DefaultLengthVector - 2 do: [ :kk | 
				y := ((states at: kk) bitAnd: Mt19937UpperMask) bitOr: ((states at: kk + 1) bitAnd: Mt19937LowerMask).
				states
					at: kk
					put:
						(((states at: kk + PeriodParameter - DefaultLengthVector + 1) bitXor: (y bitShift: 1 negated))
							bitXor: (tempArray at: (y bitAnd: 16r1) + 1)) ].
			y := ((states at: DefaultLengthVector) bitAnd: Mt19937UpperMask) bitOr: ((states at: 1) bitAnd: Mt19937LowerMask).
			states
				at: DefaultLengthVector
				put: ((states at: PeriodParameter) bitXor: ((y bitShift: 1) bitXor: (tempArray at: (y bitAnd: 16r1) + 1))).
			mti := 0 ].
	y := states at: mti + 1.
	mti := mti + 1.
	y := y bitXor: (y bitShift: 11 negated).
	y := y bitXor: ((y bitShift: 7 negated) bitAnd: TemperingMaskB).
	y := y bitXor: ((y bitShift: 15 negated) bitAnd: TemperingMaskC).
	y := y bitXor: (y bitShift: 18 negated).
	^ y! !

!MersenneTwisterRandom methodsFor: 'accessing' stamp: 'SergeStinckwich 3/10/2015 13:22'!
peek
	"Answer the same as next without changing state"
	
	^ self copy next
	! !

RandomGenerator subclass: #PMRandom
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random'!

!PMRandom commentStamp: 'hfm 3/24/2009 09:19' prior: 0!
Uses Park and Miller's 'Minimum Standard' Congruential generator. See 'Numerical Recipes in C', 2nd Edition; Press, Flannery, Teukolsky and Vetterling; Cambridge University Press, 1992.

PMRandom new next!

!PMRandom methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
next
    "Private - Answer a pseudo-Random floating point number between 0 and 1.
    Uses Park and Miller's 'Minimum Standard' congruential generator
    (see 'Numerical Recipes in C', 2nd Edition; Press, Flannery,
    Teukolsky and Vetterling; Cambridge University Press, 1992)."

	^ self nextFloat! !

!PMRandom methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
nextFloat
    "Private - Answer a pseudo-Random floating point number between 0 and 1.
    Uses Park and Miller's 'Minimum Standard' congruential generator
    (see 'Numerical Recipes in C', 2nd Edition; Press, Flannery,
    Teukolsky and Vetterling; Cambridge University Press, 1992)."

    seed := self peekSeed.
    ^ seed / 16r7FFFFFFF asFloat ! !

!PMRandom methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
peek
    "Private - Answer a pseudo-Random floating point number between 0 and 1.
    Uses Park and Miller's 'Minimum Standard' congruential generator."

    ^ self peekSeed / 16r7FFFFFFF asFloat
! !

!PMRandom methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
peekSeed
    "Private - Answer the next Integer seed calculated using Park and Miller's 'Minimum Standard'
    congruential generator. The seed is in the range 0..16r7FFFFFFF."

    ^ ( seed * ( 7 raisedToInteger: 5 ) ) rem: 16r7FFFFFFF
! !

!PMRandom methodsFor: 'accessing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
seed: anInteger
    "Set the seed of the Random stream to anInteger (or 1 if zero)."

    seed := anInteger max: 1
! !

!RandomGenerator class methodsFor: 'testing' stamp: 'SergeStinckwich 9/17/2014 16:43'!
chiSquare: range repeating: anInteger
	"Run a 'chi-square' test on a random number generator, 
      over a range of integers given by range, repeating anInteger times.  
      Answer with an Array	 containing the result of the chi-square test 
      (which should be the same as the range), and the upper and lower 
      bounds for 'good' randomness (assuming that anInteger
      is at least 10 * range)."
	"LCRandom chiSquare: 100 repeating: 4000"
	
	| array rand ave |
    array := Array new: range withAll: 0.
	rand := self new.
    anInteger timesRepeat: [  
            | temp |
            temp := (rand next * range) truncated + 1.
     		array at: temp put: (array at: temp) + 1].
			ave := 0.
			array do: [ :each | ave := ave + (each * each)].
			^ Array	with: ( range - ( 2 * range sqrt ) )
						with: ( ( ave * range ) / anInteger) - anInteger asFloat
						with: ( range + ( 2 * range sqrt ) )
! !

!RandomGenerator class methodsFor: 'instance creation' stamp: 'StephaneDucasse 3/20/2015 08:51'!
new
 	" Answer a new instance of the receiver "

	^ ( self seed: Time millisecondClockValue ) initialize! !

!RandomGenerator class methodsFor: 'instance creation' stamp: 'StephaneDucasse 3/20/2015 08:51'!
seed: anInteger
	"Anwer a new Random stream with the initial seed anInteger."

	^ self basicNew seed: anInteger; yourself! !

!RandomGenerator methodsFor: 'testing' stamp: 'StephaneDucasse 3/20/2015 08:54'!
atEnd
	"Answer whether the receiver is at its end - never true for a stream of Random numbers"

	^ false
! !

!RandomGenerator methodsFor: 'accessing' stamp: 'StephaneDucasse 3/20/2015 08:55'!
contents
	"Answer all of the objects in the collection accessed by the receiver.
	Implementation Notes: Random streams are infinite,  so there is no implementation possible."

	^ self shouldNotImplement
! !

!RandomGenerator methodsFor: 'testing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
isReadable
	"Answer whether the receiver can be read from 
	(i.e. it implements the gettableStream protocol)."

	^ true
! !

!RandomGenerator methodsFor: 'testing' stamp: 'DanielUber 6/7/2012 14:08:38.23'!
isWriteable
	"Answer whether the receiver can be written to 
	(i.e. it implements the puttableStream protocol)."

	^ false
! !

!RandomGenerator methodsFor: 'accessing' stamp: 'StephaneDucasse 3/20/2015 08:54'!
next
 	"Answer a pseudo-Random number"
	
	^ self subclassResponsibility! !

!RandomGenerator methodsFor: 'accessing' stamp: 'StephaneDucasse 3/20/2015 08:54'!
next: anInteger
	"Answer an Array of anInteger pseudo-random Floats between 0 and 1. "

	^ ( 1 to: anInteger ) 
			collect: [ :i | self next ]
! !

!RandomGenerator methodsFor: 'accessing' stamp: 'StephaneDucasse 3/20/2015 08:54'!
peek
	" Answer a pseudo-Random number generated from the next seed,  but do not advance down the stream (i.e. self peek = self peek). "

	^ self subclassResponsibility! !

!RandomGenerator methodsFor: 'accessing' stamp: 'StephaneDucasse 3/20/2015 08:54'!
seed
    "Returns the instance var of the receiver for external manipulation"

    ^ seed! !

!RandomGenerator methodsFor: 'accessing' stamp: 'StephaneDucasse 3/20/2015 08:54'!
seed: anObject
    "Sets the instance var of the receiver with anObject. (No checking is done)"

    seed := anObject! !

Random subclass: #CombinedRandomGenerator
	instanceVariableNames: 'random seedA seedB'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random'!

!CombinedRandomGenerator commentStamp: 'StephaneDucasse 3/20/2015 08:50' prior: 0!
This Combined Random Number Generator is based on the algorithm described by PIERRE L'ECUYER in "Efficient and Portable Combined Random Number Generators" [Communications of the ACM, June 19, Volume 31, Number 6, pp. 742-749, references p.774]. Taking into account its two-dimensional behaviour (from abovementioned article), this generator is suitable to produce the pairs of consecutive numbers.
For the first linear congruential generator (generator A):
m = 2147483563; a = 40014; q = 53668; r = 12211.
For the second linear congruential generator (generator B):
m = 2147483399; a = 40692; q = 52774; r = 3791.

To produce initial seedA (for the first generator)  the method #nextInt: 2147483562 of Random is used; to produce seedB (for the second)  - the method #nextInt: 2147483398. Corresponding seeds are represented as Floats. The result of work of two generators (the next seedA and seedB) are combined.

Developed by Konstantin Nizheradze <konsnizher@gmail.com> 

Instance Variables:
	random	<Random>
	seedA	<Number>
	seedB	<Number>!

!CombinedRandomGenerator methodsFor: 'initialize' stamp: 'SergeStinckwich 3/23/2015 22:41'!
initialize
	"comment stating purpose of message"
	super initialize.
	random := Random new.
	seedA := (random nextInt: 2147483562) asFloat.
	seedB := (random nextInt: 2147483398) asFloat

	

	! !

!CombinedRandomGenerator methodsFor: 'accessing' stamp: 'KonstantinNizheradze 1/29/2013 12:39'!
next
	"Combine seedA and seedB to produce new seed. Seed is in the interval [0, 1]. "

	 seed := (self nextValueA) - (self nextValueB).
	 seed < 1 ifTrue: [seed := seed + 2.147483562e9].
	 seed := seed *  4.656613057391769e-10.
	^ seed
	
	! !

!CombinedRandomGenerator methodsFor: 'private' stamp: 'KonstantinNizheradze 1/15/2013 13:14'!
nextValueA
	"Evaluate next value of seedA using m and a of generatorA"

	| lo hi aLoRHi |
	hi := (seedA quo: 53668.0) asFloat.
	lo := seedA - (hi * 53668.0).  " = seed rem: q"  
	aLoRHi := ( 40014.0 * lo) - ( 12211.0 * hi).
	seedA := (aLoRHi > 0.0)
		ifTrue:  [aLoRHi]
		ifFalse: [aLoRHi + 2.147483563e9].
	^ seedA! !

!CombinedRandomGenerator methodsFor: 'private' stamp: 'KonstantinNizheradze 1/15/2013 13:14'!
nextValueB
	"Evaluate next value of seedB using m, a, q and r of generatorB"

	| lo hi aLoRHi |
	hi := (seedB quo: 52774.0) asFloat.
	lo := seedB - (hi * 52774.0).  " = seed rem: q"  
	aLoRHi := (40692.0 * lo) - (3791.0 * hi).
	seedB := (aLoRHi > 0.0)
		ifTrue:  [aLoRHi]
		ifFalse: [aLoRHi +  2.147483399e9].
	^ seedB! !

Random subclass: #ExplicitInverseCongruentialGenerator
	instanceVariableNames: 'p nextN nextValue nextModInv'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Random'!

!ExplicitInverseCongruentialGenerator commentStamp: 'StephaneDucasse 3/20/2015 08:50' prior: 0!
An ExplicitInverseCongruentialGenerator is an explicit inversive congruential generator, constructed according to "Good random number generators are (not so) easy to find" by P. Hellekalek (1998) and extended euclidean algorithm.
Developed by Konstantin Nizheradze <konsnizher@gmail.com> 

Instance Variables
	nextN:		<Object>
	nextValue:		<Object>
	p:		<Object>

nextN
	- next number of the sequence of random numbers. It is also the parameter at extended euclidean algorythm

nextValue
	- next modulo inverse value, calculated by extended euclidean algorythm

p
	- parameter at extended euclidean algorythm
!

!ExplicitInverseCongruentialGenerator methodsFor: 'initialization' stamp: 'StephaneDucasse 3/18/2015 22:05'!
initialize
	"Initialize No using Random and define the modulo (p)"

	| random |
	super initialize.
	p := 16r7FFFFFFF asFloat.
	random := Random new.
	nextN := random nextValue.
	! !

!ExplicitInverseCongruentialGenerator methodsFor: 'accessing' stamp: 'StephaneDucasse 3/18/2015 22:05'!
next
	"This method is used by nextInt:, answer a random Float in the interval [0 to 1]"
		
	nextModInv := self nextRandomValue / p.
 	 ^ nextModInv
	
	  
	! !

!ExplicitInverseCongruentialGenerator methodsFor: 'private' stamp: 'KonstantinNizheradze 12/9/2014 12:27'!
nextRandomValue
	"Calculate the next modulo inverse value for n+1"

	| rem newrem inv newinv transrem transinv quotient |
	inv := 0.
	newinv := 1.
	rem := p.
	newrem := nextN.
	[newrem ~= 0] whileTrue: [quotient := rem // newrem.
								transrem := newrem.
								newrem := rem \\ newrem.
								rem := transrem.
								transinv := newinv.
								newinv := inv - (quotient * newinv).
								inv := transinv].
	(inv < 0) ifTrue: [inv := inv + p].						
	nextValue := inv.
	nextN := nextN + 1.
	^nextValue
	! !

!ExplicitInverseCongruentialGenerator methodsFor: 'tests' stamp: 'StephaneDucasse 3/18/2015 22:05'!
testNextValue: anInteger
	"This is to test the principle of modulo inverse"

	| rem newrem inv newinv transrem transinv quotient |
	inv := 0.
	newinv := 1.
	rem := 240.
	newrem := anInteger.
	[newrem ~= 0] whileTrue: [quotient := rem // newrem.
								transrem := newrem.
								newrem := rem \\ newrem.
								rem := transrem.
								transinv := newinv.
								newinv := inv - (quotient * newinv).
								inv := transinv].
	^ inv
	! !
MersenneTwisterRandom initialize!
